/*
 *
 * Device Drivers E!Kit-1100 specific pcmcia routines.
 *
 * Copyright 2004 Device Drivers Limited
 *
 * ########################################################################
 *
 *  This program is free software; you can distribute it and/or modify it
 *  under the terms of the GNU General Public License (Version 2) as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * ########################################################################
 *
 * 
 */
#include <linux/module.h>
#include <linux/init.h>
#include <linux/config.h>
#include <linux/delay.h>
#include <linux/ioport.h>
#include <linux/kernel.h>
#include <linux/tqueue.h>
#include <linux/timer.h>
#include <linux/mm.h>
#include <linux/proc_fs.h>
#include <linux/version.h>
#include <linux/types.h>

#include <pcmcia/version.h>
#include <pcmcia/cs_types.h>
#include <pcmcia/cs.h>
#include <pcmcia/ss.h>
#include <pcmcia/bulkmem.h>
#include <pcmcia/cistpl.h>
#include <pcmcia/bus_ops.h>
#include "cs_internal.h"

#include <asm/io.h>
#include <asm/irq.h>
#include <asm/system.h>

#include <asm/au1000.h>
#include <asm/au1000_pcmcia.h>

typedef volatile struct
{
	/* 00 */ unsigned long cf0;
	/* 04 */ unsigned long cf1;
	/* 08 */ unsigned long usb0;
	/* 0C */ unsigned long usb1;
	/* 10 */ unsigned long sys;
	/* 14 */ unsigned long gpio3;
	/* 18 */ unsigned long rsv;
	/* 1C */ unsigned long config;

} CPLD;
#define PCMCIA_MAX_SOCK 1
#define PCMCIA_NUM_SOCKS (PCMCIA_MAX_SOCK+1)

static CPLD * const cpld = (CPLD *)0xB8000000;
static int tps2223;

static void tps2223_write(int data)
{
	int i;
	
	tps2223 = data;
	
	for(i = 0; i < 11; i++)
	{
		if(data & 0x400)
			outl(0x20000, SYS_OUTPUTSET);
		else
			outl(0x20000, SYS_OUTPUTCLR);
		au_sync_delay(2);
		outl(0x40000, SYS_OUTPUTSET);
		au_sync_delay(2);
		outl(0x40000, SYS_OUTPUTCLR);
		au_sync_delay(2);
		data <<= 1;
	}
	outl(0x80000, SYS_OUTPUTSET);
	au_sync_delay(2);
	outl(0x40000, SYS_OUTPUTSET);
	au_sync_delay(2);
	outl(0x40000, SYS_OUTPUTCLR);
	au_sync_delay(2);
	outl(0x80000, SYS_OUTPUTCLR);
	au_sync_delay(2);
}

static int tps2223_read(void)
{
	return tps2223;
}

static int ekit1100_pcmcia_init(struct pcmcia_init *init)
{
	// CFA_EN      = 0
	// CFA_RST     = 1
	cpld->cf0 = 0x00400000;
	// CFB_EN      = 0
	// CFB_RST     = 1
	cpld->cf1 = 0x00400000;
	// VCCA & VCCB = 0V
	// VPPA & VPPB = 0V
	tps2223_write(0x0100);
	
	return PCMCIA_NUM_SOCKS;
}

static int ekit1100_pcmcia_shutdown(void)
{
	// CFA_EN      = 0
	// CFA_RST     = 1
	cpld->cf0 = 0x00400000;
	// CFB_EN      = 0
	// CFB_RST     = 1
	cpld->cf1 = 0x00400000;
	// VCCA & VCCB = 0V
	// VPPA & VPPB = 0V
	tps2223_write(0x0100);
	
	return 0;
}

static int ekit1100_pcmcia_socket_state(unsigned sock, struct pcmcia_state *state)
{
	u32 inserted;
	unsigned char vs;
	
	if(sock > PCMCIA_MAX_SOCK) return -1;
	
	state->ready = 0;
	state->vs_Xv = 0;
	state->vs_3v = 0;
	state->detect = 0;
	
	if(sock == 0)
	{
		vs = (cpld->cf0 & 0x0C) >> 2;
		inserted = (cpld->cf0 & 0x03);
	}
	else
	{
		vs = (cpld->cf1 & 0x0C) >> 2;
		inserted = (cpld->cf1 & 0x03);
	}
	
	DEBUG(KERN_DEBUG "ekit1100 socket %d: inserted %d, vs %d\n", sock, inserted, vs);
	
	if(inserted == 3)
	{
		switch(vs)
		{
			case 0: /* 5V */
				break;
			case 1: /* 3V */
			case 3:
				state->vs_3v=1;
				break;
			default:
				/* return without setting 'detect' */
				printk(KERN_ERR "ekit1100 bad VS (%d)\n", vs);
				return -1;
		}
		state->detect = 1;
		state->ready = 1;
	}
	else
	{
		/* if the card was previously inserted and then ejected,
		 * we should turn off power to it
		 */
		if((sock == 0) && ((cpld->cf0 & 0x00400000) == 0))
		{
			cpld->cf0 = 0x00400000;
			tps2223_write(tps2223_read() & 0xFDF0);
		}
		else if((sock == 1) && ((cpld->cf1 & 0x00400000) == 0))
		{
			cpld->cf1 = 0x00400000;
			tps2223_write(tps2223_read() & 0xFB0F);
		}
	}
	
	state->bvd1 = 1;
	state->bvd2 = 1;
	state->wrprot = 0; 
	
	return 1;
}


static int ekit1100_pcmcia_get_irq_info(struct pcmcia_irq_info *info)
{
	if(info->sock > PCMCIA_MAX_SOCK) return -1;
	
	if(info->sock == 0)
	{
		info->irq = AU1000_GPIO_0;
	}
	else 
	{
		info->irq = AU1000_GPIO_1;
	}
	
	return 0;
}


static int ekit1100_pcmcia_configure_socket(const struct pcmcia_configure *configure)
{
	u16 pwr;
	int sock = configure->sock;
	
	if(sock > PCMCIA_MAX_SOCK) return -1;
	
	DEBUG(KERN_DEBUG "socket %d Vcc %dV Vpp %dV, reset %d\n", 
			sock, configure->vcc, configure->vpp, configure->reset);
	
	/* pcmcia reg was set to zero at init time. Be careful when
	 * initializing a socket not to wipe out the settings of the 
	 * other socket.
	 */
	if(sock == 0)
	{
		switch(configure->vcc)
		{
			case 0:  /* VCC 0V */
				tps2223_write((tps2223_read() & 0xFFF3) | 0x0000);
				break;
			case 33: /* VCC 3.3V */
				tps2223_write((tps2223_read() & 0xFFF3) | 0x0004);
				break;
			case 50: /* VCC 5V */
				tps2223_write((tps2223_read() & 0xFFF3) | 0x0008);
				break;
			default: /* what's this ? */
				printk(KERN_ERR "%s: bad Vcc %d\n", __FUNCTION__, configure->vcc);
				break;
		}
	}
	else
	{
		switch(configure->vcc)
		{
			case 0:  /* VCC 0V */
				tps2223_write((tps2223_read() & 0xFF3F) | 0x0000);
				break;
			case 33: /* VCC 3.3V */
				tps2223_write((tps2223_read() & 0xFF3F) | 0x0080);
				break;
			case 50: /* VCC 5V */
				tps2223_write((tps2223_read() & 0xFF3F) | 0x0040);
				break;
			default: /* what's this ? */
				printk(KERN_ERR "%s: bad Vcc %d\n", __FUNCTION__, configure->vcc);
				break;
		}
	}
	
	au_sync_delay(300);
	
	if(sock == 0)
	{
		if(!configure->reset)
		{
			cpld->cf0 = 0x00A00000;
			au_sync_delay(100);
		}
		else
		{
			cpld->cf0 = 0x00400000;
			au_sync_delay(100);
		}
	}
	else
	{
		if(!configure->reset)
		{
			cpld->cf1 = 0x00A00000;
			au_sync_delay(100);
		}
		else
		{
			cpld->cf1 = 0x00400000;
			au_sync_delay(100);
		}
	}
	
	return 0;
}

struct pcmcia_low_level ekit1100_pcmcia_ops =
{ 
	ekit1100_pcmcia_init,
	ekit1100_pcmcia_shutdown,
	ekit1100_pcmcia_socket_state,
	ekit1100_pcmcia_get_irq_info,
	ekit1100_pcmcia_configure_socket
};
